unit Excel;

{ Excel DDE for Delphi 1 by Paul TOTH <tothpaul@multimania.com> }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DDEML, StartApp;

type
  TExcel = class(TComponent)
  private
    { Dclarations private }
    fHConv:HConv;
    fTopic:string;
    fHTopic:HSZ;
    fItem:string;
    fHItem:HSZ;
    fData:string;
    fMacro:string;
    fHData:HDDEData;
    fhTrans:Longint;
    fOpenMac:String;
    fReadOnly:String;
    EOnError:TNotifyEvent;
    EOnDisconnect:TNotifyEvent;
    fExcelPath:string;
    Function GetActive:boolean;
    Procedure SetActive(Value:boolean);
    Procedure SetTopic(Value:string);
    Procedure SetItem(Value:string);
    Procedure SetData(Value:string);
    Procedure SetMacro(Value:string);
  protected
    { Dclarations protected }
    Function Exec(Conv:HConv;Macro:String):boolean;
  public
    { Dclarations public }
    Constructor Create(AOwner:TComponent); override;
    Destructor Destroy; override;
    Function Connect:boolean;
  published
    { Dclarations published }
    Property Active:Boolean read GetActive write SetActive;
    Property Topic:string read fTopic write SetTopic;
    Property Item:String read fItem write SetItem;
    Property Data:String read fData write SetData;
    Property Macro:String read fMacro write SetMacro;
    Property OpenMac:String read fOpenMac write fOpenMac;
    Property ReadOnly:string read fReadOnly write fReadOnly;
    Property OnError:TNotifyEvent read EOnError write EOnError;
    property OnDisconnect:TNotifyEvent read EOnDisconnect write EOnDisconnect;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional', [TExcel]);
end;

Var
 HDdeCallBack:TFarProc;
 DdeHandle:LongInt;
 HExcel:HSZ;
 xlStatus:(xlDDEError,xlReady,xlBusy,xlComplet,xlDisconnect,xlError,xlOther);

Function DdeCallBack(
 CallType, Fmt: Word; Conv: HConv; hsz1, hsz2: HSZ; Data: HDDEData; Data1, Data2: Longint
): HDDEData; export;
begin
 Result:=0;
 Case CallType of
   XTYP_XACT_COMPLETE : xlStatus:=xlComplet;
   XTYP_DISCONNECT    : xlStatus:=XlDisconnect;
   XTYP_ERROR         : xlStatus:=xlError;
   else                 xlStatus:=xlOther;
 end;
end;

{ TExcel }
Constructor TExcel.Create(AOwner:TComponent);
 begin
  inherited Create(AOwner);
  fHConv:=0;
  fHItem:=0;
  fHData:=0;
  fOpenMac:='OUVRIR'; { this is the Excel macro to open files (OUVRIR=OPEN in french) }
  fReadOnly:='VRAI';  { this is the value of Boolen for ReadOnly (VRAI=TRUE in French) }
  fExcelPath:='excel.exe';
 end;

Destructor TExcel.Destroy;
 begin
  Active:=False;
  Topic:='';
  Item:='';
  Inherited Destroy;
 end;

 Function TExcel.GetActive:boolean;
  begin
   if xlStatus=xlDisconnect then begin
    xlStatus:=xlError;
    Active:=False;
    if Assigned(EOnDisconnect) then EOnDisconnect(Self);
   end;
   Result:=fHConv<>0;
  end;

 Procedure TExcel.SetActive(Value:boolean);
  Var
   Conv:HConv;
   Good:boolean;
  begin
   if Value<>Active then
    if Value then begin
    { Check Excel }
     If (not StartApplication('XLMAIN',nil,fExcelPath)) then exit;
    { Connect }
     fHConv:=DdeConnect(DdeHandle,HExcel,fHTopic,nil);
     if fHConv=0 then begin
      if fHTopic=0 then exit; { null topic, there is nothing more to do ! }
      Conv:=DdeConnect(DdeHandle,HExcel,0,nil); { Try a null topic }
       if Conv=0 then exit; { unabled to connect }
       Good:=Exec(Conv,'['+OpenMac+'("'+fTopic+'";;'+fReadOnly+')]'); { open the topic }
      DdeDisconnect(Conv); { close connection }
      if not good then exit; { unabled to open doc }
      fHConv:=DdeConnect(DdeHandle,HExcel,fHTopic,nil); { retry connection }
      if fHConv=0 then exit; { definitively not !}
     end;
    end else begin
     if xlStatus=xlBusy then DdeAbandonTransaction(DdeHandle,fhConv,fhTrans);
     DdeDisConnect(fHConv);
     xlStatus:=xlReady;
     fHConv:=0;
    end;
  end;

 Procedure TExcel.SetTopic(Value:string);
  begin
   if fTopic=Value then exit;
   fTopic:=Value;
   if Value='' then
    DdeFreeStringHandle(DdeHandle,fhTopic)
   else begin
    Value:=Value+#0;
    fHTopic:=DdeCreateStringHandle(DdeHandle,@Value[1],0);
    if fHTopic=0 then exit;
   end;
  end;

 Procedure TExcel.SetItem(Value:string);
  begin
   if fItem=Value then exit;
   fItem:=Value;
   if Value='' then
    DdeFreeStringHandle(DdeHandle,fhItem)
   else begin
    Value:=Value+#0;
    fhItem:=DdeCreateStringHandle(DdeHandle,@Value[1],0);
    if fHItem=0 then exit;
   end;
  end;

Procedure TExcel.SetData(Value:string);
 begin
  Active:=True;
  xlStatus:=xlBusy;
  Value[Length(value)+1]:=#0;
  if DdeClientTransaction(@Value[1],Length(Value)+1,fhConv,fhItem,CF_TEXT,XTYP_POKE,TIMEOUT_ASYNC,@fhTrans)<>0 then begin
   Repeat Application.ProcessMessages until xlStatus<>xlBusy;
  end;
  if xlStatus<>xlComplet then begin
   fData:=Value;
   if xlStatus=xlDisconnect then
    if Assigned(EOnDisconnect) then begin
     Active:=False;
     EOnDisconnect(Self);
     exit;
    end;
    if Assigned(EOnError) then EOnError(Self);
  end;
  xlStatus:=xlReady;
 end;

Procedure TExcel.SetMacro(Value:string);
 begin
  Active:=True;
  if not Exec(fhConv,Value) then begin
   fMacro:=Value;
   if Assigned(EOnError) then EOnError(Self);
  end;
 end;

Function TExcel.Exec(Conv:HConv;Macro:String):boolean;
 begin
  xlStatus:=xlBusy;
  Macro:=Macro+#0;
  if DdeClientTransaction(@Macro[1],Length(Macro),Conv,0,CF_TEXT,XTYP_EXECUTE,TIMEOUT_ASYNC,nil)<>0 then begin
   Repeat Application.ProcessMessages until xlStatus<>xlBusy;
  end;
  Result:=(xlStatus=xlComplet);
  if xlStatus=xlDisconnect then begin
   Active:=False;
   if Assigned(EOnDisconnect) then EOnDisconnect(Self);
  end;
  xlStatus:=xlReady;
 end;

Function TExcel.Connect:boolean;
 begin
  If Not Active then Active:=True;
  Result:=Active; 
 end;

Procedure DoneDDE; far;
 begin
  DdeFreeStringHandle(DdeHandle,HExcel);
  DdeUninitialize(DdeHandle);
 end;

Procedure InitDDE;
 begin
  xlStatus:=xlDDEError;
  HDdeCallBack:= MakeProcInstance(@DdeCallBack, HInstance);
  DdeHandle:=0;
  if DdeInitialize(DdeHandle,TCallBack(HDdeCallBack),APPCLASS_STANDARD,0)<>0 then exit;
  HExcel:=DdeCreateStringHandle(DdeHandle,'Excel',0);
  AddExitProc(DoneDDE);
  xlStatus:=xlReady;
 end;

begin
 InitDDE;
end.
